require 'spec_helper'

describe TMDB::Trailer do

  before(:each) do
    stub_request(:any, /.*Movie\.browse.*countries=br.*/).
      to_return(:body => fixture("browse_br.json"))

    stub_request(:any, /.*Movie\.browse.*countries=br.*&page=[^1].*?/).
      to_return(:body => fixture("browse_no_result.json"))

    stub_request(:any, /.*Movie\.browse.*countries=nz.*/).
      to_return(:body => fixture("browse_nz.json"))

    stub_request(:any, /.*Movie\.browse.*countries=nz.*&page=[^1].*?/).
      to_return(:body => fixture("browse_no_result.json"))

    stub_request(:any, /.*Movie\.browse.*countries=cz.*&page=1.*/).
      to_return(:body => fixture("browse_cz_1.json"))

    stub_request(:any, /.*Movie\.browse.*countries=cz.*&page=2.*/).
      to_return(:body => fixture("browse_cz_2.json"))

    stub_request(:any, /.*Movie\.browse.*countries=cz.*&page=3.*?/).
      to_return(:body => fixture("browse_no_result.json"))

    stub_request(:any, /.*Movie\.getInfo.*/).to_return(:body => fixture("get_info_br.json"))
    #reset mongo DB database
    @trailer = TMDB::Trailer.new("br","test")
    @trailer.coll.remove
    #reset pstore page counters
    pstore = PStore.new("test.pstore")
    pstore.transaction do
      pstore["br"] = 1
      pstore["cz"] = 1
      pstore["nz"] = 1
    end
  end

  context "get countries current page" do
    it "should display correct page" do
      page = @trailer.get_page
      page.should eq(1)
    end

    it "should grab json response from a given url" do
      page = @trailer.get_page
      js = @trailer.get_movies(page)
      js.length.should eq(37)
    end

    it "should make api calls and move to next pages until no result" do
      country = "br"
      @trailer.populate
      pstore = PStore.new("test.pstore")
      pstore.transaction do
        @expected = pstore[country] + 0
      end
      @trailer.current_page.should eq(@expected)
    end

    it "should store results to mongodb" do
      doc = {}
      size_before = @trailer.coll.count
      page = @trailer.get_page
      js = @trailer.get_movies(page)
      @trailer.store_result(js)
      @trailer.coll.count.should eq(size_before + js.length)
    end

    it "get_extra_info should work on movie" do
      id = 20268 #id for Last Stop 174
      trailer, genres , keywords, year = TMDB::Trailer.new("br","test").get_extra_info(id)
      trailer.should eq("http://www.youtube.com/watch?v=2IOFiuzFFqw")
      genres.include?("Drama").should be_true
      keywords.length.should eq(0)
      year.should eq("2008")
    end

    it "should not store duplicate result" do
      coll_length = []
      (1..3).each do 
        @trailer.populate
        coll_length << @trailer.coll.count
      end
      count = @trailer.coll.count
      non_repeat_values = coll_length.reject {|x| x == count}
      non_repeat_values.should == []
      no_duplicate = non_repeat_values.length == 0
      no_duplicate.should be_true
    end

    it "should be able to report the progress of populating the database" do
      @trailer.populate
      progress = @trailer.report
      progress.should match(/37 out of 37/)
    end

    it "should be able to count the number of movies a country has on tmdb" do
      # tmdb only have 37 brazilian films that has min_votes of 1
      total_movies = @trailer.total_movies
      total_movies.should == 37
    end

    it "should store the correct page on pstore" do
      pstore = PStore.new("test.pstore")
      @trailer.populate
      pstore.transaction do
        pstore["br"].should == 2
      end

      @trailer = TMDB::Trailer.new("cz","test")
      @trailer.coll.remove()
      @trailer.populate
      pstore.transaction do
        pstore["cz"].should == 3
      end
    end

    it "should be able to populate database for other countries" do
      @trailer = TMDB::Trailer.new("nz","test")
      @trailer.coll.remove()
      @trailer.populate
      @trailer.coll.count.should == 42
      progress = @trailer.report 
      progress.should match(/42 out of 42/)

      @trailer = TMDB::Trailer.new("cz","test")
      @trailer.coll.remove()
      @trailer.populate
      @trailer.coll.count.should == 60
      progress = @trailer.report 
      progress.should match(/60 out of 60/)
    end

    it "should fully populate database for a country" do
      WebMock.allow_net_connect!
      @trailer = TMDB::Trailer.new("ro","test")
      @trailer.coll.remove()
      @trailer.populate
      @trailer.coll.count.should >= 28
    end
  end
end
